@@ -27,8 +27,6 @@
 #include <gtk/gtk.h>
 #include <glib/gi18n-lib.h>
 
-#include <libmissioncontrol/mission-control.h>
-
 #include <libempathy/empathy-contact-manager.h>
 #include <libempathy/empathy-account-manager.h>
 #include <libempathy/empathy-contact-list.h>
@@ -340,7 +338,8 @@ can_add_contact_to_account (EmpathyAccount *account,
 		return FALSE;
 
 	contact_manager = empathy_contact_manager_dup_singleton ();
-	result = empathy_contact_manager_get_flags_for_connection (contact_manager, connection) & EMPATHY_CONTACT_LIST_CAN_ADD;
+	result = empathy_contact_manager_get_flags_for_connection (
+		contact_manager, connection) & EMPATHY_CONTACT_LIST_CAN_ADD;
 	g_object_unref (contact_manager);
 
 	return result;
