@@ -42,6 +42,7 @@
 #include <extensions/extensions.h>
 
 #include "empathy-dispatcher.h"
+#include "empathy-handler.h"
 #include "empathy-utils.h"
 #include "empathy-tube-handler.h"
 #include "empathy-account-manager.h"
@@ -55,6 +56,8 @@
 #define GET_PRIV(obj) EMPATHY_GET_PRIV (obj, EmpathyDispatcher)
 typedef struct
 {
+  gboolean dispose_has_run;
+
   EmpathyAccountManager *account_manager;
   /* connection to connection data mapping */
   GHashTable *connections;
@@ -64,33 +67,38 @@ typedef struct
 
   /* channels which the dispatcher is listening "invalidated" */
   GList *channels;
+  GPtrArray *array;
+
+  /* main handler */
+  EmpathyHandler *handler;
+
+  /* extra handlers */
+  GList *handlers;
 
   GHashTable *request_channel_class_async_ids;
 } EmpathyDispatcherPriv;
 
-static void empathy_dispatcher_client_handler_iface_init (gpointer g_iface,
-  gpointer g_iface_data);
+static GList *
+empathy_dispatcher_get_channels (EmpathyHandler *handler,
+    gpointer user_data);
 
-G_DEFINE_TYPE_WITH_CODE (EmpathyDispatcher,
-    empathy_dispatcher,
-    G_TYPE_OBJECT,
-    G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_DBUS_PROPERTIES,
-      tp_dbus_properties_mixin_iface_init);
-    G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_CLIENT, NULL);
-    G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_CLIENT_HANDLER,
-      empathy_dispatcher_client_handler_iface_init);
-  );
+static gboolean
+empathy_dispatcher_handle_channels (EmpathyHandler *handler,
+    const gchar *account_path,
+    const gchar *connection_path,
+    const GPtrArray *channels,
+    const GPtrArray *requests_satisfied,
+    guint64 timestamp,
+    GHashTable *handler_info,
+    gpointer user_data,
+    GError **error);
 
-static const gchar *empathy_dispatcher_interfaces[] = {
-  TP_IFACE_CLIENT_HANDLER,
-  NULL
-};
+G_DEFINE_TYPE (EmpathyDispatcher, empathy_dispatcher, G_TYPE_OBJECT);
 
 enum
 {
   PROP_INTERFACES = 1,
-  PROP_CHANNEL_FILTER,
-  PROP_CHANNELS
+  PROP_HANDLER,
 };
 
 enum
@@ -171,6 +179,10 @@ typedef struct
   gpointer user_data;
 } FindChannelRequest;
 
+static void
+empathy_dispatcher_call_create_channel (EmpathyDispatcher *dispatcher,
+    DispatcherRequestData *request_data);
+
 static DispatchData *
 new_dispatch_data (TpChannel *channel,
                    GObject *channel_wrapper)
@@ -865,7 +877,7 @@ dispatcher_constructor (GType type,
                         GObjectConstructParam *construct_params)
 {
   GObject *retval;
-  TpDBusDaemon *dbus;
+  EmpathyDispatcherPriv *priv;
 
   if (dispatcher != NULL)
     return g_object_ref (dispatcher);
@@ -876,18 +888,46 @@ dispatcher_constructor (GType type,
   dispatcher = EMPATHY_DISPATCHER (retval);
   g_object_add_weak_pointer (retval, (gpointer) &dispatcher);
 
-  dbus = tp_dbus_daemon_dup (NULL);
+  priv = GET_PRIV (dispatcher);
 
-  g_assert (tp_dbus_daemon_request_name (dbus,
-    DISPATCHER_BUS_NAME, TRUE, NULL));
-  dbus_g_connection_register_g_object (tp_get_bus (),
-    DISPATCHER_OBJECT_PATH, retval);
+  if (priv->handler == NULL)
+    priv->handler = empathy_handler_new (NULL, NULL, NULL);
 
-  DEBUG ("Registering at '%s'", DISPATCHER_OBJECT_PATH);
+  empathy_handler_set_handle_channels_func (priv->handler,
+    empathy_dispatcher_handle_channels,
+    dispatcher);
+
+  empathy_handler_set_channels_func (priv->handler,
+    empathy_dispatcher_get_channels,
+    dispatcher);
 
   return retval;
 }
 
+static void
+dispatcher_dispose (GObject *object)
+{
+  EmpathyDispatcherPriv *priv = GET_PRIV (object);
+  GList *l;
+
+  if (priv->dispose_has_run)
+    return;
+
+  priv->dispose_has_run = TRUE;
+
+  for (l = priv->handlers ; l != NULL; l = g_list_next (l))
+    g_object_unref (G_OBJECT (l->data));
+
+  g_list_free (priv->handlers);
+  priv->handlers = NULL;
+
+  if (priv->handler != NULL)
+    g_object_unref (priv->handler);
+  priv->handler = NULL;
+
+  G_OBJECT_CLASS (empathy_dispatcher_parent_class)->dispose (object);
+}
+
 static void
 dispatcher_finalize (GObject *object)
 {
@@ -935,6 +975,26 @@ dispatcher_finalize (GObject *object)
   g_hash_table_destroy (priv->outstanding_classes_requests);
 }
 
+static void
+dispatcher_set_property (GObject *object,
+  guint property_id,
+  const GValue *value,
+  GParamSpec *pspec)
+{
+  EmpathyDispatcher *dispatcher = EMPATHY_DISPATCHER (object);
+  EmpathyDispatcherPriv *priv = GET_PRIV (dispatcher);
+
+  switch (property_id)
+    {
+      case PROP_HANDLER:
+        priv->handler = g_value_dup_object (value);
+        break;
+      default:
+        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+        break;
+    }
+}
+
 static void
 dispatcher_get_property (GObject *object,
   guint property_id,
@@ -946,37 +1006,9 @@ dispatcher_get_property (GObject *object,
 
   switch (property_id)
     {
-      case PROP_INTERFACES:
-        g_value_set_boxed (value, empathy_dispatcher_interfaces);
+      case PROP_HANDLER:
+        g_value_set_object (value, priv->handler);
         break;
-      case PROP_CHANNEL_FILTER:
-        {
-          GPtrArray *filters = g_ptr_array_new ();
-          GHashTable *filter = g_hash_table_new (NULL, NULL);
-
-          g_ptr_array_add (filters, filter);
-
-          g_value_set_boxed (value, filters);
-          break;
-        }
-      case PROP_CHANNELS:
-        {
-          GPtrArray *accounts;
-          GList *l;
-
-          accounts = g_ptr_array_new ();
-
-          for (l = priv->channels; l != NULL; l = g_list_next (l))
-            {
-              TpProxy *channel = TP_PROXY (l->data);
-
-              g_ptr_array_add (accounts,
-                g_strdup (tp_proxy_get_object_path (channel)));
-            }
-
-          g_value_set_boxed (value, accounts);
-          break;
-        }
       default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
         break;
@@ -989,53 +1021,19 @@ empathy_dispatcher_class_init (EmpathyDispatcherClass *klass)
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
   GParamSpec *param_spec;
 
-  static TpDBusPropertiesMixinPropImpl client_props[] = {
-    { "Interfaces", "interfaces", NULL },
-    { NULL }
-  };
-  static TpDBusPropertiesMixinPropImpl client_handler_props[] = {
-    { "HandlerChannelFilter", "channel-filter", NULL },
-    { "HandledChannels", "channels", NULL },
-    { NULL }
-  };
-  static TpDBusPropertiesMixinIfaceImpl prop_interfaces[] = {
-    { TP_IFACE_CLIENT,
-      tp_dbus_properties_mixin_getter_gobject_properties,
-      NULL,
-      client_props
-    },
-    { TP_IFACE_CLIENT_HANDLER,
-      tp_dbus_properties_mixin_getter_gobject_properties,
-      NULL,
-      client_handler_props
-    },
-    { NULL }
-  };
-
+  object_class->dispose = dispatcher_dispose;
   object_class->finalize = dispatcher_finalize;
   object_class->constructor = dispatcher_constructor;
 
   object_class->get_property = dispatcher_get_property;
+  object_class->set_property = dispatcher_set_property;
 
-  param_spec = g_param_spec_boxed ("interfaces", "interfaces",
-    "Available D-Bus interfaces",
-    G_TYPE_STRV,
-    G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
-  g_object_class_install_property (object_class, PROP_INTERFACES, param_spec);
-
-  param_spec = g_param_spec_boxed ("channel-filter", "channel-filter",
-    "Filter for channels this handles",
-    TP_ARRAY_TYPE_CHANNEL_CLASS_LIST,
-    G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+  param_spec = g_param_spec_object ("handler", "handler",
+    "The main Telepathy Client Hander object",
+    EMPATHY_TYPE_HANDLER,
+    G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_CONSTRUCT_ONLY);
   g_object_class_install_property (object_class,
-    PROP_CHANNEL_FILTER, param_spec);
-
-  param_spec = g_param_spec_boxed ("channels", "channels",
-    "List of channels we're handling",
-    EMPATHY_ARRAY_TYPE_OBJECT,
-    G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
-  g_object_class_install_property (object_class,
-    PROP_CHANNELS, param_spec);
+    PROP_HANDLER, param_spec);
 
   signals[OBSERVE] =
     g_signal_new ("observe",
@@ -1069,10 +1067,6 @@ empathy_dispatcher_class_init (EmpathyDispatcherClass *klass)
 
 
   g_type_class_add_private (object_class, sizeof (EmpathyDispatcherPriv));
-
-  klass->dbus_props_class.interfaces = prop_interfaces;
-  tp_dbus_properties_mixin_class_init (object_class,
-    G_STRUCT_OFFSET (EmpathyDispatcherClass, dbus_props_class));
 }
 
 static void
@@ -1109,6 +1103,26 @@ empathy_dispatcher_init (EmpathyDispatcher *dispatcher)
     g_direct_equal);
 }
 
+EmpathyDispatcher *
+empathy_dispatcher_new (const gchar *name,
+  GPtrArray *filters,
+  GStrv capabilities)
+{
+  g_assert (dispatcher == NULL);
+  EmpathyHandler *handler;
+  EmpathyDispatcher *ret;
+
+  handler = empathy_handler_new (name, filters, capabilities);
+
+  ret = EMPATHY_DISPATCHER (
+    g_object_new (EMPATHY_TYPE_DISPATCHER,
+      "handler", handler,
+      NULL));
+  g_object_unref (handler);
+
+  return ret;
+}
+
 EmpathyDispatcher *
 empathy_dispatcher_dup_singleton (void)
 {
@@ -1123,7 +1137,8 @@ dispatcher_request_failed (EmpathyDispatcher *dispatcher,
   EmpathyDispatcherPriv *priv = GET_PRIV (dispatcher);
   ConnectionData *conn_data;
 
-  conn_data = g_hash_table_lookup (priv->connections, request_data->connection);
+  conn_data = g_hash_table_lookup (priv->connections,
+      request_data->connection);
   if (request_data->cb != NULL)
     request_data->cb (NULL, error, request_data->user_data);
 
@@ -1246,12 +1261,34 @@ dispatcher_request_channel_cb (TpConnection *connection,
 static void
 dispatcher_request_channel (DispatcherRequestData *request_data)
 {
-  tp_cli_connection_call_request_channel (request_data->connection, -1,
-    request_data->channel_type,
-    request_data->handle_type,
-    request_data->handle,
-    TRUE, dispatcher_request_channel_cb,
-    request_data, NULL, G_OBJECT (request_data->dispatcher));
+  if (tp_proxy_has_interface_by_id (TP_PROXY (request_data->connection),
+      TP_IFACE_QUARK_CONNECTION_INTERFACE_REQUESTS))
+    {
+      /* Extend the request_data to be a valid request */
+      g_assert (request_data->request == NULL);
+      request_data->request = tp_asv_new (
+        TP_IFACE_CHANNEL ".ChannelType",
+          G_TYPE_STRING, request_data->channel_type,
+        TP_IFACE_CHANNEL ".TargetHandleType",
+          G_TYPE_UINT, request_data->handle_type,
+        NULL);
+
+      if (request_data->handle_type != TP_HANDLE_TYPE_NONE)
+        tp_asv_set_uint32 (request_data->request,
+          TP_IFACE_CHANNEL ".TargetHandle", request_data->handle);
+
+      empathy_dispatcher_call_create_channel (request_data->dispatcher,
+        request_data);
+    }
+  else
+    {
+      tp_cli_connection_call_request_channel (request_data->connection, -1,
+        request_data->channel_type,
+        request_data->handle_type,
+        request_data->handle,
+        TRUE, dispatcher_request_channel_cb,
+        request_data, NULL, G_OBJECT (request_data->dispatcher));
+    }
 }
 
 void
@@ -1309,7 +1346,8 @@ dispatcher_chat_with_contact_id_cb (EmpathyTpContactFactory *factory,
     }
   else
     {
-      empathy_dispatcher_chat_with_contact (contact, data->callback, data->user_data);
+      empathy_dispatcher_chat_with_contact (contact, data->callback,
+          data->user_data);
     }
 
   g_object_unref (data->dispatcher);
@@ -1425,6 +1463,16 @@ dispatcher_create_channel_cb (TpConnection *connect,
     request_data, object_path, properties, error);
 }
 
+static void
+empathy_dispatcher_call_create_channel (EmpathyDispatcher *dispatcher,
+    DispatcherRequestData *request_data)
+{
+  tp_cli_connection_interface_requests_call_create_channel (
+    request_data->connection, -1,
+    request_data->request, dispatcher_create_channel_cb, request_data, NULL,
+    G_OBJECT (request_data->dispatcher));
+}
+
 void
 empathy_dispatcher_create_channel (EmpathyDispatcher *dispatcher,
                                    TpConnection *connection,
@@ -1463,10 +1511,7 @@ empathy_dispatcher_create_channel (EmpathyDispatcher *dispatcher,
   connection_data->outstanding_requests = g_list_prepend
     (connection_data->outstanding_requests, request_data);
 
-  tp_cli_connection_interface_requests_call_create_channel (
-    request_data->connection, -1,
-    request_data->request, dispatcher_create_channel_cb, request_data, NULL,
-    G_OBJECT (request_data->dispatcher));
+  empathy_dispatcher_call_create_channel (dispatcher, request_data);
 }
 
 static gboolean
@@ -1783,7 +1828,7 @@ empathy_dispatcher_find_requestable_channel_classes_async
 
   /* append another request for this connection */
   request = g_slice_new0 (FindChannelRequest);
-  request->dispatcher = dispatcher;
+  request->dispatcher = g_object_ref (dispatcher);
   request->channel_type = g_strdup (channel_type);
   request->handle_type = handle_type;
   request->connection = connection;
@@ -1797,17 +1842,28 @@ empathy_dispatcher_find_requestable_channel_classes_async
     request, GUINT_TO_POINTER (source_id));
 }
 
-static void
-empathy_dispatcher_handle_channels (TpSvcClientHandler *self,
+static GList *
+empathy_dispatcher_get_channels (EmpathyHandler *handler,
+  gpointer user_data)
+{
+  EmpathyDispatcher *dispatcher = EMPATHY_DISPATCHER (user_data);
+  EmpathyDispatcherPriv *priv = GET_PRIV (dispatcher);
+
+  return priv->channels;
+}
+
+static gboolean
+empathy_dispatcher_handle_channels (EmpathyHandler *handler,
     const gchar *account_path,
     const gchar *connection_path,
     const GPtrArray *channels,
     const GPtrArray *requests_satisfied,
     guint64 timestamp,
     GHashTable *handler_info,
-    DBusGMethodInvocation *context)
+    gpointer user_data,
+    GError **error)
 {
-  EmpathyDispatcher *dispatcher = EMPATHY_DISPATCHER (self);
+  EmpathyDispatcher *dispatcher = EMPATHY_DISPATCHER (user_data);
   EmpathyDispatcherPriv *priv = GET_PRIV (dispatcher);
   int i;
   EmpathyAccount *account;
@@ -1821,10 +1877,9 @@ empathy_dispatcher_handle_channels (TpSvcClientHandler *self,
       connection_path);
   if (connection == NULL)
     {
-      GError error = { TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
-        "Invalid connection argument" };
-      dbus_g_method_return_error (context, &error);
-      return;
+      g_set_error_literal (error, TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
+        "Invalid connection argument");
+      return FALSE;
     }
 
   for (i = 0; i < channels->len ; i++)
@@ -1840,15 +1895,43 @@ empathy_dispatcher_handle_channels (TpSvcClientHandler *self,
         connection, object_path, properties);
     }
 
-  tp_svc_client_handler_return_from_handle_channels (context);
+  return TRUE;
 }
 
-static void
-empathy_dispatcher_client_handler_iface_init (gpointer g_iface,
-  gpointer g_iface_data)
+
+EmpathyHandler *
+empathy_dispatcher_add_handler (EmpathyDispatcher *dispatcher,
+    const gchar *name,
+    GPtrArray *filters,
+    GStrv capabilities)
 {
-  TpSvcClientHandlerClass *klass = (TpSvcClientHandlerClass *) g_iface;
+  EmpathyDispatcherPriv *priv = GET_PRIV (dispatcher);
+  EmpathyHandler *handler;
 
-  tp_svc_client_handler_implement_handle_channels (klass,
-    empathy_dispatcher_handle_channels);
+  handler = empathy_handler_new (name, filters, capabilities);
+  priv->handlers = g_list_prepend (priv->handlers, handler);
+
+  /* Only set the handle_channels function, the Channel property on the main
+   * handler will always report all dispatched channels even if they came from
+   * a different Handler */
+  empathy_handler_set_handle_channels_func (handler,
+    empathy_dispatcher_handle_channels,
+    dispatcher);
+
+  return handler;
+}
+
+void
+empathy_dispatcher_remove_handler (EmpathyDispatcher *dispatcher,
+  EmpathyHandler *handler)
+{
+  EmpathyDispatcherPriv *priv = GET_PRIV (dispatcher);
+  GList *h;
+
+  h = g_list_find (priv->handlers, handler);
+  g_return_if_fail (h != NULL);
+
+  priv->handlers = g_list_delete_link (priv->handlers, h);
+
+  g_object_unref (handler);
 }
